// implementation of static repository

use crate::admin::domain::*;
use crate::admin::state::AdminRegistry;
use log::{debug, warn};
use serde_json::{json, Value};

struct MyStaticRepository {
    /// In Memory Storage for Example.
    content: Vec<Value>,
    /// ID Counter for content.
    next_id: usize,
}

impl MyStaticRepository {
    pub fn new() -> Self {
        MyStaticRepository {
            next_id: 12,
            content: vec![
                json!({"id": 1, "name": "Strange", "age": 150, "level": "master" }),
                json!({"id": 2, "name": "Adam", "age": 12, "powers": 8}),
                json!({"id": 3, "name": "Tony", "age": 42, "powers": 0}),
                json!({"id": 4, "name": "Rex", "age": 72}),
                json!({"id": 5, "name": "Justin", "age": 46}),
                json!({"id": 6, "name": "Reacher", "age": 39, "level": "adept", "powers": 0}),
                json!({"id": 7, "name": "Arnold", "age": 7}),
                json!({"id": 8, "name": "Eight", "age": 8}),
                json!({"id": 9, "name": "Nine", "age": 9}),
                json!({"id": 10, "name": "Ten", "age": 10}),
                json!({"id": 11, "name": "Eleven", "age": 11}),
            ],
        }
    }
}

impl AdminRepository for MyStaticRepository {
    fn get(&self, model: &AdminModel, id: LookupKey) -> Option<RepositoryItem> {
        if let LookupKey::Integer(id) = id {
            let item = self.content.get(id - 1).cloned().unwrap();
            Some(RepositoryItem {
                detail_url: Some(format!(
                    "{}/detail/{}",
                    model.admin_url,
                    item.get("id").unwrap()
                )),
                fields: item,
            })
        } else {
            warn!("Got non-integer lookup key: {}", id);
            None
        }
    }

    fn list(&self, model: &AdminModel) -> RepositoryList {
        RepositoryList::List {
            values: self
                .content
                .clone()
                .into_iter()
                .map(|item| RepositoryItem {
                    detail_url: Some(format!(
                        "{}/detail/{}",
                        model.admin_url,
                        item.get("id").unwrap()
                    )),
                    fields: item,
                })
                .collect(),
        }
    }

    fn info(&self, _model: &AdminModel) -> RepositoryInfo {
        RepoInfo {
            name: "My Static Repository",
            lookup_key: "id",
            display_list: &["id", "name", "level", "age"],
            fields: &["name", "level", "age", "powers"],
        }
        .into()
    }

    fn create(&mut self, model: &AdminModel, mut data: Value) -> Option<RepositoryItem> {
        debug!("Asked to create: {}", data);

        let new_id = self.next_id;
        self.next_id += 1;

        data["id"] = Value::Number(new_id.into());

        debug!("I create: {}", data);

        // Push the data into the repository
        self.content.push(data.clone());

        // Return the newly created item
        Some(RepositoryItem {
            detail_url: Some(format!("{}/detail/{}", model.admin_url, new_id)),
            fields: data,
        })
    }

    fn update(&mut self, model: &AdminModel, id: LookupKey, data: Value) -> Option<RepositoryItem> {
        debug!("I would now update: {}, {}", id, data);

        // First, find the index of the item to update
        let item_index = self.content.iter().position(|item| {
            if let Some(item_id) = item.get("id") {
                match id {
                    LookupKey::Integer(i) => item_id == &i,
                    LookupKey::String(ref s) => item_id == s,
                }
            } else {
                false
            }
        });

        // Then, update the item if found
        if let Some(index) = item_index {
            let item = &mut self.content[index];
            *item = data.clone();

            if let Some(item_id) = item.get("id") {
                return Some(RepositoryItem {
                    detail_url: Some(format!("{}/detail/{}", model.admin_url, item_id)),
                    fields: data,
                });
            }
        }

        None
    }

    fn delete(&mut self, _model: &AdminModel, id: LookupKey) -> Option<Value> {
        debug!("Would delete: {}", id);

        let item_index = self.content.iter().position(|item| {
            if let Some(item_id) = item.get("id") {
                match id {
                    LookupKey::Integer(i) => item_id == &i,
                    LookupKey::String(ref s) => item_id == s,
                }
            } else {
                false
            }
        });

        if let Some(index) = item_index {
            // Remove and return the item
            Some(self.content.remove(index))
        } else {
            None
        }
    }
}

pub fn register_example(registry: &mut AdminRegistry) {
    let app_key = registry.register_app("Example App");
    let repo = MyStaticRepository::new();
    let model_config = AdminModelConfig {
        app_key: app_key,
        name: "ExampleModel".to_owned(),
    };
    let model_result = registry.register_model(model_config, repo);
    match model_result {
        Err(err) => panic!("{}", err),
        _ => (),
    }
}
